<div class="cadquery-overlay">
    <button class="cadquery-credit">Modeled with CadQuery</button>
    <div class="cadquery-overlay-dropdown-content">
        <p class="footer">Exported as SVG.</p>
    </div>
</div>
