.. raw:: html

    <div class="cadquery-container" style="margin-bottom: 24px">
        <div class="cadquery-container-model">
            <div class="cadquery-svg">
                {{svg_document|indent(8)}}
            </div>
            <div class="cadquery-overlay">
                <button class="cadquery-credit">Modeled with CadQuery</button>
                <div class="cadquery-overlay-dropdown-content">
                    <p class="footer">Exported as SVG.</p>
                </div>
            </div>
        </div>

{% if include_source %}
.. code-block:: python

    {{script_source|indent(4)}}
{% endif %}

.. raw:: html

    </div>
